import path from "path";
import fs from "fs";
import grabHostNames from "../../utils/grab-host-names";
import apiUpdateUser from "../../functions/api/users/api-update-user";
import { UpdateUserFunctionReturn } from "../../types";

type Param = {
    key?: string;
    database?: string;
    updatedUserId: string | number;
    payload: { [s: string]: any };
    user_id?: boolean;
};

/**
 * # Update User
 */
export default async function updateUser({
    key,
    payload,
    database,
    user_id,
    updatedUserId,
}: Param): Promise<UpdateUserFunctionReturn> {
    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        global.DSQL_USE_LOCAL
    ) {
        /** @type {import("../../types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema: import("../../types").DSQL_DatabaseSchemaType | undefined;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiUpdateUser({
            payload: payload,
            dbFullName: DSQL_DB_NAME,
            updatedUserId,
            dbSchema,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        const reqPayload = JSON.stringify({
            payload,
            database,
            updatedUserId,
        });

        const httpsRequest = scheme.request(
            {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization:
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY ||
                        key,
                },
                port,
                hostname: host,
                path: `/api/user/${
                    user_id || grabedHostNames.user_id
                }/update-user`,
            },

            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";

                response.on("data", function (chunk) {
                    str += chunk;
                });

                response.on("end", function () {
                    resolve(JSON.parse(str));
                });

                response.on("error", (err) => {
                    reject(err);
                });
            }
        );
        httpsRequest.write(reqPayload);
        httpsRequest.end();
    });

    return httpResponse as UpdateUserFunctionReturn;
}
